@@ -47,23 +47,5 @@ dbind_method_call_reentrant (DBusConnection *cnx,
                              const char *arg_types,
                              ...);
 
-dbus_bool_t
-dbind_emit_signal_va (DBusConnection *cnx,
-                      const char *path,
-                      const char *interface,
-                      const char *signal,
-                      DBusError *opt_error,
-                      const char *arg_types,
-                      va_list args);
-
-dbus_bool_t
-dbind_emit_signal (DBusConnection *cnx,
-                   const char *path,
-                   const char *interface,
-                   const char *signal,
-                   DBusError *opt_error,
-                   const char *arg_types,
-                   ...);
-
 void dbind_set_timeout (int timeout);
 #endif /* _DBIND_H_ */
